@@ -4,18 +4,21 @@ import warnings
 
 import numpy as np
 
-from numpy.testing import (assert_almost_equal, assert_array_equal,
-        TestCase, run_module_suite, assert_allclose, assert_equal, assert_)
-from scipy.interpolate import (splrep, splev,
-        KroghInterpolator, krogh_interpolate,
-        BarycentricInterpolator, barycentric_interpolate,
-        PiecewisePolynomial, piecewise_polynomial_interpolate,
-        approximate_taylor_polynomial, pchip, PchipInterpolator,
-        Akima1DInterpolator)
+from numpy.testing import (
+    assert_almost_equal, assert_array_equal, assert_array_almost_equal,
+    TestCase, run_module_suite, assert_allclose, assert_equal, assert_,
+    assert_raises)
+
+from scipy.interpolate import (
+    KroghInterpolator, krogh_interpolate,
+    BarycentricInterpolator, barycentric_interpolate,
+    approximate_taylor_polynomial, pchip, PchipInterpolator,
+    pchip_interpolate, Akima1DInterpolator, CubicSpline)
 from scipy._lib.six import xrange
 
 
-def check_shape(interpolator_cls, x_shape, y_shape, deriv_shape=None, axis=0):
+def check_shape(interpolator_cls, x_shape, y_shape, deriv_shape=None, axis=0,
+                extra_args={}):
     np.random.seed(1234)
 
     x = [-1, 0, 1]
@@ -28,7 +31,7 @@ def check_shape(interpolator_cls, x_shape, y_shape, deriv_shape=None, axis=0):
         return
 
     xi = np.zeros(x_shape)
-    yi = interpolator_cls(x, y, axis=axis)(xi)
+    yi = interpolator_cls(x, y, axis=axis, **extra_args)(xi)
 
     target_shape = ((deriv_shape or ()) + y.shape[:axis]
                     + x_shape + y.shape[axis:][1:])
@@ -36,7 +39,7 @@ def check_shape(interpolator_cls, x_shape, y_shape, deriv_shape=None, axis=0):
 
     # check it works also with lists
     if x_shape and y.size > 0:
-        interpolator_cls(list(x), list(y), axis=axis)(list(xi))
+        interpolator_cls(list(x), list(y), axis=axis, **extra_args)(list(xi))
 
     # check also values
     if xi.size > 0 and deriv_shape is None:
@@ -52,12 +55,16 @@ SHAPES = [(), (0,), (1,), (3, 2, 5)]
 
 def test_shapes():
     for ip in [KroghInterpolator, BarycentricInterpolator, pchip,
-               Akima1DInterpolator]:
+               Akima1DInterpolator, CubicSpline]:
         for s1 in SHAPES:
             for s2 in SHAPES:
                 for axis in range(-len(s2), len(s2)):
-                    yield check_shape, ip, s1, s2, None, axis
-
+                    if ip != CubicSpline:
+                        yield check_shape, ip, s1, s2, None, axis
+                    else:
+                        for bc in ['natural', 'clamped']:
+                            extra = {'bc_type': bc}
+                            yield check_shape, ip, s1, s2, None, axis, extra
 
 def test_derivs_shapes():
     def krogh_derivs(x, y, axis=0):
@@ -98,8 +105,15 @@ def test_deriv_shapes():
     def akima_antideriv(x, y, axis=0):
         return Akima1DInterpolator(x, y, axis).antiderivative()
 
+    def cspline_deriv(x, y, axis=0):
+        return CubicSpline(x, y, axis).derivative()
+
+    def cspline_antideriv(x, y, axis=0):
+        return CubicSpline(x, y, axis).antiderivative()
+
     for ip in [krogh_deriv, pchip_deriv, pchip_deriv2, pchip_deriv_inplace,
-               pchip_antideriv, pchip_antideriv2, akima_deriv, akima_antideriv]:
+               pchip_antideriv, pchip_antideriv2, akima_deriv, akima_antideriv,
+               cspline_deriv, cspline_antideriv]:
         for s1 in SHAPES:
             for s2 in SHAPES:
                 for axis in range(-len(s2), len(s2)):
@@ -114,7 +128,7 @@ def _check_complex(ip):
 
 
 def test_complex():
-    for ip in [KroghInterpolator, BarycentricInterpolator, pchip]:
+    for ip in [KroghInterpolator, BarycentricInterpolator, pchip, CubicSpline]:
         yield _check_complex, ip
 
 
@@ -309,148 +323,6 @@ class CheckBarycentric(TestCase):
         assert_almost_equal(P(self.test_xs),barycentric_interpolate(self.xs,self.ys,self.test_xs))
 
 
-class CheckPiecewise(TestCase):
-    def setUp(self):
-        self.tck = splrep([0,1,2,3,4,5], [0,10,-1,3,7,2], s=0)
-        self.test_xs = np.linspace(-1,6,100)
-        self.spline_ys = splev(self.test_xs, self.tck)
-        self.spline_yps = splev(self.test_xs, self.tck, der=1)
-        self.xi = np.unique(self.tck[0])
-        self.yi = [[splev(x, self.tck, der=j) for j in xrange(3)] for x in self.xi]
-
-    def test_construction(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi, self.yi, 3)
-
-        assert_almost_equal(P(self.test_xs), self.spline_ys)
-
-    def test_scalar(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi,3)
-
-        assert_almost_equal(P(self.test_xs[0]),self.spline_ys[0])
-        assert_almost_equal(P.derivative(self.test_xs[0],1),self.spline_yps[0])
-        assert_almost_equal(P(np.array(self.test_xs[0])),self.spline_ys[0])
-        assert_almost_equal(P.derivative(np.array(self.test_xs[0]),1),
-                            self.spline_yps[0])
-
-    def test_derivative(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi,3)
-
-        assert_almost_equal(P.derivative(self.test_xs,1),self.spline_yps)
-
-    def test_derivatives(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi,3)
-
-        m = 4
-        r = P.derivatives(self.test_xs,m)
-        #print r.shape, r
-        for i in xrange(m):
-            assert_almost_equal(P.derivative(self.test_xs,i),r[i])
-
-    def test_vector(self):
-        xs = [0, 1, 2]
-        ys = [[[0,1]],[[1,0],[-1,-1]],[[2,1]]]
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(xs,ys)
-            Pi = [PiecewisePolynomial(xs,[[yd[i] for yd in y] for y in ys])
-                for i in xrange(len(ys[0][0]))]
-
-        test_xs = np.linspace(-1,3,100)
-        assert_almost_equal(P(test_xs),
-                np.rollaxis(np.asarray([p(test_xs) for p in Pi]),-1))
-        assert_almost_equal(P.derivative(test_xs,1),
-                np.transpose(np.asarray([p.derivative(test_xs,1) for p in Pi]),
-                    (1,0)))
-
-    def test_incremental(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial([self.xi[0]], [self.yi[0]], 3)
-
-        for i in xrange(1,len(self.xi)):
-            P.append(self.xi[i],self.yi[i],3)
-        assert_almost_equal(P(self.test_xs),self.spline_ys)
-
-    def test_shapes_scalarvalue(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi,4)
-
-        assert_array_equal(np.shape(P(0)), ())
-        assert_array_equal(np.shape(P(np.array(0))), ())
-        assert_array_equal(np.shape(P([0])), (1,))
-        assert_array_equal(np.shape(P([0,1])), (2,))
-
-    def test_shapes_scalarvalue_derivative(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi,4)
-
-        n = 4
-        assert_array_equal(np.shape(P.derivative(0,1)), ())
-        assert_array_equal(np.shape(P.derivative(np.array(0),1)), ())
-        assert_array_equal(np.shape(P.derivative([0],1)), (1,))
-        assert_array_equal(np.shape(P.derivative([0,1],1)), (2,))
-
-    def test_shapes_vectorvalue(self):
-        yi = np.multiply.outer(np.asarray(self.yi),np.arange(3))
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,yi,4)
-
-        assert_array_equal(np.shape(P(0)), (3,))
-        assert_array_equal(np.shape(P([0])), (1,3))
-        assert_array_equal(np.shape(P([0,1])), (2,3))
-
-    def test_shapes_vectorvalue_1d(self):
-        yi = np.multiply.outer(np.asarray(self.yi),np.arange(1))
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,yi,4)
-
-        assert_array_equal(np.shape(P(0)), (1,))
-        assert_array_equal(np.shape(P([0])), (1,1))
-        assert_array_equal(np.shape(P([0,1])), (2,1))
-
-    def test_shapes_vectorvalue_derivative(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi, np.multiply.outer(self.yi,
-                                                               np.arange(3)),4)
-
-        n = 4
-        assert_array_equal(np.shape(P.derivative(0,1)), (3,))
-        assert_array_equal(np.shape(P.derivative([0],1)), (1,3))
-        assert_array_equal(np.shape(P.derivative([0,1],1)), (2,3))
-
-    def test_wrapper(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', category=DeprecationWarning)
-            P = PiecewisePolynomial(self.xi,self.yi)
-
-            assert_almost_equal(P(self.test_xs),
-                                piecewise_polynomial_interpolate(
-                                    self.xi, self.yi, self.test_xs))
-            assert_almost_equal(P.derivative(self.test_xs,2),
-                                piecewise_polynomial_interpolate(self.xi,
-                                                                 self.yi,
-                                                                 self.test_xs,
-                                                                 der=2))
-            assert_almost_equal(P.derivatives(self.test_xs,2),
-                                piecewise_polynomial_interpolate(self.xi,
-                                                                 self.yi,
-                                                                 self.test_xs,
-                                                                 der=[0,1]))
-
-
 class TestPCHIP(TestCase):
     def _make_random(self, npts=20):
         np.random.seed(1234)
@@ -492,5 +364,269 @@ class TestPCHIP(TestCase):
 
         assert_allclose(curve, curve1, atol=1e-14, rtol=1e-14)
 
+    def test_nag(self):
+        # Example from NAG C implementation,
+        # http://nag.com/numeric/cl/nagdoc_cl25/html/e01/e01bec.html
+        # suggested in gh-5326 as a smoke test for the way the derivatives
+        # are computed (see also gh-3453)
+        from scipy._lib.six import StringIO
+        dataStr = '''
+          7.99   0.00000E+0
+          8.09   0.27643E-4
+          8.19   0.43750E-1
+          8.70   0.16918E+0
+          9.20   0.46943E+0
+         10.00   0.94374E+0
+         12.00   0.99864E+0
+         15.00   0.99992E+0
+         20.00   0.99999E+0
+        '''
+        data = np.loadtxt(StringIO(dataStr))
+        pch = pchip(data[:,0], data[:,1])
+
+        resultStr = '''
+           7.9900       0.0000
+           9.1910       0.4640
+          10.3920       0.9645
+          11.5930       0.9965
+          12.7940       0.9992
+          13.9950       0.9998
+          15.1960       0.9999
+          16.3970       1.0000
+          17.5980       1.0000
+          18.7990       1.0000
+          20.0000       1.0000
+        '''
+        result = np.loadtxt(StringIO(resultStr))
+        assert_allclose(result[:,1], pch(result[:,0]), rtol=0., atol=5e-5)
+
+    def test_endslopes(self):
+        # this is a smoke test for gh-3453: PCHIP interpolator should not
+        # set edge slopes to zero if the data do not suggest zero edge derivatives
+        x = np.array([0.0, 0.1, 0.25, 0.35])
+        y1 = np.array([279.35, 0.5e3, 1.0e3, 2.5e3])
+        y2 = np.array([279.35, 2.5e3, 1.50e3, 1.0e3])
+        for pp in (pchip(x, y1), pchip(x, y2)):
+            for t in (x[0], x[-1]):
+                assert_(pp(t, 1) != 0)
+
+    def test_all_zeros(self):
+        x = np.arange(10)
+        y = np.zeros_like(x)
+
+        # this should work and not generate any warnings
+        with warnings.catch_warnings():
+            warnings.filterwarnings('error')
+            pch = pchip(x, y)
+
+        xx = np.linspace(0, 9, 101)
+        assert_equal(pch(xx), 0.)
+
+    def test_two_points(self):
+        # regression test for gh-6222: pchip([0, 1], [0, 1]) fails because
+        # it tries to use a three-point scheme to estimate edge derivatives,
+        # while there are only two points available.
+        # Instead, it should construct a linear interpolator.
+        x = np.linspace(0, 1, 11)
+        p = pchip([0, 1], [0, 2])
+        assert_allclose(p(x), 2*x, atol=1e-15)
+
+    def test_pchip_interpolate(self):
+        assert_array_almost_equal(
+            pchip_interpolate([1,2,3], [4,5,6], [0.5], der=1),
+            [1.])
+
+        assert_array_almost_equal(
+            pchip_interpolate([1,2,3], [4,5,6], [0.5], der=0),
+            [3.5])
+
+        assert_array_almost_equal(
+            pchip_interpolate([1,2,3], [4,5,6], [0.5], der=[0, 1]),
+            [[3.5], [1]])
+
+
+class TestCubicSpline(object):
+    @staticmethod
+    def check_correctness(S, bc_start='not-a-knot', bc_end='not-a-knot',
+                          tol=1e-14):
+        """Check that spline coefficients satisfy the continuity and boundary
+        conditions."""
+        x = S.x
+        c = S.c
+        dx = np.diff(x)
+        dx = dx.reshape([dx.shape[0]] + [1] * (c.ndim - 2))
+        dxi = dx[:-1]
+
+        # Check C2 continuity.
+        assert_allclose(c[3, 1:], c[0, :-1] * dxi**3 + c[1, :-1] * dxi**2 +
+                        c[2, :-1] * dxi + c[3, :-1], rtol=tol, atol=tol)
+        assert_allclose(c[2, 1:], 3 * c[0, :-1] * dxi**2 +
+                        2 * c[1, :-1] * dxi + c[2, :-1], rtol=tol, atol=tol)
+        assert_allclose(c[1, 1:], 3 * c[0, :-1] * dxi + c[1, :-1],
+                        rtol=tol, atol=tol)
+
+        # Check that we found a parabola, the third derivative is 0.
+        if x.size == 3 and bc_start == 'not-a-knot' and bc_end == 'not-a-knot':
+            assert_allclose(c[0], 0, rtol=tol, atol=tol)
+            return
+
+        # Check periodic boundary conditions.
+        if bc_start == 'periodic':
+            assert_allclose(S(x[0], 0), S(x[-1], 0), rtol=tol, atol=tol)
+            assert_allclose(S(x[0], 1), S(x[-1], 1), rtol=tol, atol=tol)
+            assert_allclose(S(x[0], 2), S(x[-1], 2), rtol=tol, atol=tol)
+            return
+
+        # Check other boundary conditions.
+        if bc_start == 'not-a-knot':
+            if x.size == 2:
+                slope = (S(x[1]) - S(x[0])) / dx[0]
+                assert_allclose(S(x[0], 1), slope, rtol=tol, atol=tol)
+            else:
+                assert_allclose(c[0, 0], c[0, 1], rtol=tol, atol=tol)
+        elif bc_start == 'clamped':
+            assert_allclose(S(x[0], 1), 0, rtol=tol, atol=tol)
+        elif bc_start == 'natural':
+            assert_allclose(S(x[0], 2), 0, rtol=tol, atol=tol)
+        else:
+            order, value = bc_start
+            assert_allclose(S(x[0], order), value, rtol=tol, atol=tol)
+
+        if bc_end == 'not-a-knot':
+            if x.size == 2:
+                slope = (S(x[1]) - S(x[0])) / dx[0]
+                assert_allclose(S(x[1], 1), slope, rtol=tol, atol=tol)
+            else:
+                assert_allclose(c[0, -1], c[0, -2], rtol=tol, atol=tol)
+        elif bc_end == 'clamped':
+            assert_allclose(S(x[-1], 1), 0, rtol=tol, atol=tol)
+        elif bc_end == 'natural':
+            assert_allclose(S(x[-1], 2), 0, rtol=tol, atol=tol)
+        else:
+            order, value = bc_end
+            assert_allclose(S(x[-1], order), value, rtol=tol, atol=tol)
+
+    def check_all_bc(self, x, y, axis):
+        deriv_shape = list(y.shape)
+        del deriv_shape[axis]
+        first_deriv = np.empty(deriv_shape)
+        first_deriv.fill(2)
+        second_deriv = np.empty(deriv_shape)
+        second_deriv.fill(-1)
+        bc_all = [
+            'not-a-knot',
+            'natural',
+            'clamped',
+            (1, first_deriv),
+            (2, second_deriv)
+        ]
+        for bc in bc_all[:3]:
+            S = CubicSpline(x, y, axis=axis, bc_type=bc)
+            self.check_correctness(S, bc, bc)
+
+        for bc_start in bc_all:
+            for bc_end in bc_all:
+                S = CubicSpline(x, y, axis=axis, bc_type=(bc_start, bc_end))
+                self.check_correctness(S, bc_start, bc_end, tol=2e-14)
+
+    def test_general(self):
+        x = np.array([-1, 0, 0.5, 2, 4, 4.5, 5.5, 9])
+        y = np.array([0, -0.5, 2, 3, 2.5, 1, 1, 0.5])
+        for n in [2, 3, x.size]:
+            self.check_all_bc(x[:n], y[:n], 0)
+
+            Y = np.empty((2, n, 2))
+            Y[0, :, 0] = y[:n]
+            Y[0, :, 1] = y[:n] - 1
+            Y[1, :, 0] = y[:n] + 2
+            Y[1, :, 1] = y[:n] + 3
+            self.check_all_bc(x[:n], Y, 1)
+
+    def test_periodic(self):
+        for n in [2, 3, 5]:
+            x = np.linspace(0, 2 * np.pi, n)
+            y = np.cos(x)
+            S = CubicSpline(x, y, bc_type='periodic')
+            self.check_correctness(S, 'periodic', 'periodic')
+
+            Y = np.empty((2, n, 2))
+            Y[0, :, 0] = y
+            Y[0, :, 1] = y + 2
+            Y[1, :, 0] = y - 1
+            Y[1, :, 1] = y + 5
+            S = CubicSpline(x, Y, axis=1, bc_type='periodic')
+            self.check_correctness(S, 'periodic', 'periodic')
+
+    def test_periodic_eval(self):
+        x = np.linspace(0, 2 * np.pi, 10)
+        y = np.cos(x)
+        S = CubicSpline(x, y, bc_type='periodic')
+        assert_almost_equal(S(1), S(1 + 2 * np.pi), decimal=15)
+
+    def test_dtypes(self):
+        x = np.array([0, 1, 2, 3], dtype=int)
+        y = np.array([-5, 2, 3, 1], dtype=int)
+        S = CubicSpline(x, y)
+        self.check_correctness(S)
+
+        y = np.array([-1+1j, 0.0, 1-1j, 0.5-1.5j])
+        S = CubicSpline(x, y)
+        self.check_correctness(S)
+
+        S = CubicSpline(x, x ** 3, bc_type=("natural", (1, 2j)))
+        self.check_correctness(S, "natural", (1, 2j))
+
+        y = np.array([-5, 2, 3, 1])
+        S = CubicSpline(x, y, bc_type=[(1, 2 + 0.5j), (2, 0.5 - 1j)])
+        self.check_correctness(S, (1, 2 + 0.5j), (2, 0.5 - 1j))
+
+    def test_small_dx(self):
+        rng = np.random.RandomState(0)
+        x = np.sort(rng.uniform(size=100))
+        y = 1e4 + rng.uniform(size=100)
+        S = CubicSpline(x, y)
+        self.check_correctness(S, tol=1e-13)
+
+    def test_incorrect_inputs(self):
+        x = np.array([1, 2, 3, 4])
+        y = np.array([1, 2, 3, 4])
+        xc = np.array([1 + 1j, 2, 3, 4])
+        xn = np.array([np.nan, 2, 3, 4])
+        xo = np.array([2, 1, 3, 4])
+        yn = np.array([np.nan, 2, 3, 4])
+        y3 = [1, 2, 3]
+        x1 = [1]
+        y1 = [1]
+
+        assert_raises(ValueError, CubicSpline, xc, y)
+        assert_raises(ValueError, CubicSpline, xn, y)
+        assert_raises(ValueError, CubicSpline, x, yn)
+        assert_raises(ValueError, CubicSpline, xo, y)
+        assert_raises(ValueError, CubicSpline, x, y3)
+        assert_raises(ValueError, CubicSpline, x[:, np.newaxis], y)
+        assert_raises(ValueError, CubicSpline, x1, y1)
+
+        wrong_bc = [('periodic', 'clamped'),
+                    ((2, 0), (3, 10)),
+                    ((1, 0), ),
+                    (0., 0.),
+                    'not-a-typo']
+
+        for bc_type in wrong_bc:
+            assert_raises(ValueError, CubicSpline, x, y, 0, bc_type, True)
+
+        # Shapes mismatch when giving arbitrary derivative values:
+        Y = np.c_[y, y]
+        bc1 = ('clamped', (1, 0))
+        bc2 = ('clamped', (1, [0, 0, 0]))
+        bc3 = ('clamped', (1, [[0, 0]]))
+        assert_raises(ValueError, CubicSpline, x, Y, 0, bc1, True)
+        assert_raises(ValueError, CubicSpline, x, Y, 0, bc2, True)
+        assert_raises(ValueError, CubicSpline, x, Y, 0, bc3, True)
+
+        # periodic condition, y[-1] must be equal to y[0]:
+        assert_raises(ValueError, CubicSpline, x, y, 0, 'periodic', True)
+
+
 if __name__ == '__main__':
     run_module_suite()
